module Adorn
  class AbstractPresenter

    # Macro used to create an instance method that wraps the underlying
    # object passed to the decorator
    #
    # @param [Symbol]
    # @return [Method] Adorn instance.
    def self.presents(name)
      define_method(name) { @object }
    end

    # Check both the context and the object before passing exception along
    # @return [Method, StandardError]
    def method_missing(method, *args, &block)
      @context.send(method, *args, &block) if @context.respond_to?(method)
      @object.send(method, *args, &block) if @object.respond_to?(method)
    rescue => e
      super(method, *args, &block)
    end

    def respond_to_missing?(method,*)
      method =~ /#{method.to_s}/ || super
    end

    private
    def initialize(*);end
  end
end
