# Provides interface for subclassing decorators. Methods on the decorated object will
# be passed through the method that is defined using the ::presents method. Every object must
# provide an object to decorate and a context object, options are passed through having no effect
# on the presented object. The context object is implicit, when using the helper method #presenting
# provided by Adorn::Helpers
#
# ex. Interface
#
# require 'adorn'
# class WelcomePresernter < Adorn::Presenter
#   Adorn::Delegate.extensions { {with: [ImportantModule, NeededBehavior]} }
#
#   presents :welcomer
#
#   def welcome
#     welomer.english_greeting # => 'Hello there!'
#   end
# end

module Adorn
  class Presenter < AbstractPresenter
    include Adorn::Delegate


    #@param [Object, Object, Hash]
    #@return [Adorn::Presenter]
    def initialize(*args)
      object, context, options = args
      @object                  = ObjectDecorator.new(object)
      @context                 = DisplayContext.new(context)
      super(@object, @context, options)
    end
  end
end
